package com.diss.omppapp.services;

import com.diss.omppapp.entities.AnswerEntity;
import com.diss.omppapp.entities.QuestionEntity;
import com.diss.omppapp.repositories.AnswerEntityRepository;
import com.diss.omppapp.repositories.QuestionEntityRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

@Component
public class MinutePaperService {

    @Autowired
    private QuestionEntityRepository questionEntityRepository;

    @Autowired
    AnswerEntityRepository answerEntityRepository;

    public List<QuestionEntity> getQuestions(String username, String paperId) {
        if ((username == null || username.isEmpty()) || (paperId == null || paperId.isEmpty())) {
            return new ArrayList<>();
        }
        Optional<List<QuestionEntity>> questionEntities = questionEntityRepository.findAllByPaperIdOrderByQuestionIdAsc(getPaperId(paperId));
        return questionEntities.orElseGet(ArrayList::new);
    }

    public List<AnswerEntity> getAnswers(String username, String paperId) {
        if ((username == null || username.isEmpty()) || (paperId == null || paperId.isEmpty())) {
            return new ArrayList<>();
        }
        Optional<List<AnswerEntity>> answerEntities = answerEntityRepository.findAllByPaperIdOrderByAnswerIdAsc(getPaperId(paperId));
        return answerEntities.orElseGet(ArrayList::new);
    }

    private int getPaperId(String paperId) {
        int id = 0;

        try {
            id = Integer.parseInt(paperId);
        } catch (NumberFormatException nfe) {
            return 0;
        }
        return id;
    }

}
