package com.diss.omppapp.services;

import com.diss.omppapp.entities.AnswerEntity;
import com.diss.omppapp.entities.ClassEntity;
import com.diss.omppapp.entities.MinutePaperEntity;
import com.diss.omppapp.entities.StudentClassesEntity;
import com.diss.omppapp.repositories.AnswerEntityRepository;
import com.diss.omppapp.repositories.ClassEntityRepository;
import com.diss.omppapp.repositories.MinutePaperEntityRepository;
import com.diss.omppapp.repositories.StudentClassesEntityRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.*;

@Component
public class StudentInfoService {

    @Autowired
    private ClassEntityRepository classEntityRepository;

    @Autowired
    private MinutePaperEntityRepository minutePaperEntityRepository;

    @Autowired
    private AnswerEntityRepository answerEntityRepository;

    @Autowired
    private StudentClassesEntityRepository studentClassesEntityRepository;

    public List<ClassEntity> getClasses(String username) {
        if(username == null || username.isEmpty()) {
            return new ArrayList<>();
        }

        Optional<List<StudentClassesEntity>> classes = studentClassesEntityRepository.findAllByUsername(username);
        List<ClassEntity> response = new ArrayList<>();

        classes.ifPresent(studentClassesEntities -> studentClassesEntities.forEach(e -> {
            response.add(classEntityRepository.findByClassCode(e.classCode).get());
        }));
        return response;
    }

    public List<MinutePaperEntity> getOpenPapers(String username) {
        if(username == null || username.isEmpty()) {
            return new ArrayList<>();
        }
        List<MinutePaperEntity> result = new ArrayList<>();
        result.addAll(getDrafts(username));
        result.addAll(getUnansweredPapers(username));
        return result;
    }

    public List<MinutePaperEntity> getClassPapers(String username, String classCode, Boolean submitted) {
        if((username == null || username.isEmpty()) || (classCode == null || classCode.isEmpty())) {
            return new ArrayList<>();
        }

        List<MinutePaperEntity> result = new ArrayList<>();
        Optional<List<MinutePaperEntity>> possiblePapers = minutePaperEntityRepository.findAllByClassCode(classCode);
        Optional<List<AnswerEntity>> possibleSubmissions = answerEntityRepository.findAllByUsernameAndSubmitted(username,submitted);
        Set<Integer> paperIds = new LinkedHashSet<>();

        possibleSubmissions.ifPresent(submissions -> submissions.forEach(e -> {paperIds.add(e.getPaperId());}));

        possiblePapers.ifPresent(papers -> papers.forEach(e -> {
            if(paperIds.contains(e.getPaperId())) {
                result.add(e);
            }
        }));

        return result;
    }

    private List<MinutePaperEntity> getDrafts(String username) {
        List<MinutePaperEntity> result = new ArrayList<>();
        Optional<List<AnswerEntity>> openPapers = answerEntityRepository.findAllByUsernameAndSubmitted(username,false);

        openPapers.ifPresent(submissionEntities -> submissionEntities.forEach(e -> {
            result.add(minutePaperEntityRepository.findByPaperId(e.getPaperId()).get());
        }));
        return result;
    }

    private List<MinutePaperEntity> getUnansweredPapers(String username) {
        List<MinutePaperEntity> result = new ArrayList<>();
        Optional<List<StudentClassesEntity>> classlist = studentClassesEntityRepository.findAllByUsername(username);
        List<MinutePaperEntity> allPapers = new ArrayList<>();
        Set<Integer> paperIds = new LinkedHashSet<>();

        classlist.ifPresent(list -> {
            list.forEach(entity -> {
                minutePaperEntityRepository.findAllByClassCode(entity.classCode).ifPresent(allPapers::addAll);
            });
        });

        Optional<List<AnswerEntity>> allSubmittedAnswers = answerEntityRepository.findAllByUsernameAndSubmitted(username,true);
        allSubmittedAnswers.ifPresent(list -> {
            list.forEach(answer -> {
                paperIds.add(answer.getPaperId());
            });
        });

        allPapers.forEach(paper -> {
            if(!paperIds.contains(paper.getPaperId())) {
                result.add(paper);
            }
        });
        return result;
    }

}
